
/* Belvedere4.pde
 * 
 * Designed to run on Arduino MEGA
 *
 * This is code that runs Belvedere, a butler robot.  
 * I designed and built Belvedere to serve appetizers at parties 
 * and entertain with jokes and dancing.  He has the ability to 
 * navigate the first floor of our house while avoiding obstacles.  
 * His main body sits on an iRobot Create, which he uses as a drive system.  
 * 
 * My code is available to the public.  Please contact me if you have any questions:
 * andrew.v.wolff@gmail.com
 */



// Includes
#include "oi.h"
#include "defines.h"
#include <TimerThree.h>
#include <Wire.h>   // for compass reading
#include <FatReader.h>
#include <SdReader.h>
#include <avr/pgmspace.h>
#include "WaveUtil.h"
#include "WaveHC.h"
#include <stdio.h>
#include <EEPROM.h>
#include <ServoTimer2.h>
#include <LiquidCrystal.h>

// initialize the LCD library with the numbers of the interface pins
LiquidCrystal lcd(40, 41, 42, 43, 44, 45);

// Constants
const byte RESET_SONG = 0;
const byte START_SONG = 1;
const byte BUMP_SONG = 2;
const byte END_SONG = 3;
const byte zero = 0;

SdReader card;    // This object holds the information for the card
FatVolume vol;    // This holds the information for the partition on the card
FatReader root;   // This holds the information for the filesystem on the card
FatReader f;      // This holds the information for the file we're play
WaveHC wave;      // This is the only wave (audio) object, since we will only play one at a time

ServoTimer2 servoBowTie; 

// Global variables
volatile unsigned int timer_cnt = 0;
volatile int timer_on = 0;
volatile int sensors_flag = 0;
volatile int sensors_index = 0;
volatile int sensors_in[Sen6Size];
volatile int sensors[Sen6Size];
volatile int distance = 0;
volatile int angle = 0;
volatile int heading = 0;   // compass heading
volatile int compassAddress = 0x42 >> 1;
volatile int countFlag1 = 1;

volatile int delayJokeCount;    //  These are the counters that trigger occasional events
volatile int delayOfferCount;
volatile int delayCommentCount;
volatile int delayDanceCount;
volatile int delayObstacleCount;
volatile int delayWelcomeCount;
volatile int timeStationary;  // This is a counter for keeping track of the time that belvedere is stationary

unsigned long updateDelayCheck;
unsigned long loopDelayCheck;

// Wavefront Global Variables
int robot_x = Dock_x;
int robot_y = Dock_y;
int target_x = Dock_x;
int target_y = Dock_y;
int old_target_x = 0;
int old_target_y = 0;
int oldState = 1;  //1 is up, 2 is right, 3 is down, and 4 is left (clockwise)
int newState = 1;
int x = 0;  //map location
int y = 0;  //Nothing location
int updatePositionFlag = 0;
int waveCounter = 0;
int minimum_node = 250;
int min_node_location = 0;
int reset_min = 250;  //anything above this number is a special item, ie a wall or robot


// Maps  //X is vertical, Y is horizontal

int waveMap[Max_x][Max_y] = {
//{  0 , 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 , 9 , 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22
  { 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255 },//0
  { 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255 },//1
  { 255,255,255,255,255,255,255,255,255, 0 , 0 , 0 , 0 , 0 ,255,255,255,255,255,255,255,255,255 },//2
  { 255,255,255,255,255,255,255,255,255, 0 , 0 , 0 , 0 , 0 ,255,255,255,255,255,255,255,255,255 },//3
  { 255,255,255,255,255,255,255,255,255, 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 ,255,255,255,255,255,255 },//4
  { 255,255,255,255,255,255,255,255,255,255,255, 0 ,255,255,255,255, 0 , 0 , 0 , 0 , 0 ,255,255 },//5
  { 255,255,255,255,255,255,255,255,255,255,255, 0 ,255,255,255,255, 0 , 0 , 0 , 0 , 0 ,255,255 },//6
  { 255,255,255,255,255,255,255,255,255,255,255, 0 ,255,255,255,255,255,255,255,255, 0 ,255,255 },//7
  { 255, 0 , 0 ,255,255,255,255,255,255,255,255, 0 ,255,255,255,255,255,255,255,255, 0 ,255,255 },//8
  { 255, 0 , 0 , 0 ,255,255,255,255,255,255,255, 0 ,255,255,255,255,255,255,255,255, 0 ,255,255 },//9
  { 255, 0 , 0 , 0 , 0 ,255,255,255,255,255,255, 0 ,255,255,255,255,255,255,255,255, 0 ,255,255 },//10
  { 255,255,255,255, 0 ,255,255,255,255,255,255, 0 ,255,255,255,255,255,255,255,255, 0 ,255,255 },//11
  { 255,255,255,255, 0 , 0 ,255,255,255,255,255, 0 ,255,255,255,255,255,255,255,255, 0 ,255,255 },//12
  { 255,255,255,255, 0 , 0 ,255,255,255,255,255, 0 ,255,255,255,255,255,255,255,255,255,255,255 },//13
  { 255,255,255,255, 0 , 0 , 0 , 0 , 0 , 0 , 0 , 0 ,255,255,255,255,255,255,255,255,255,255,255 },//14
  { 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255 },//15
  { 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255 } //16
  
};

int roomMap[Max_x][Max_y] = {
//{  0 , 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 , 9 , 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22
  { 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255 },
  { 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255 },
  { 255,255,255,255,255,255,255,255,255, 2 , 2 , 2 , 2 , 2 ,255,255,255,255,255,255,255,255,255 },
  { 255,255,255,255,255,255,255,255,255, 2 , 2 , 2 , 2 , 2 ,255,255,255,255,255,255,255,255,255 },
  { 255,255,255,255,255,255,255,255,255, 2 , 2 , 2 , 2 , 2 , 2 , 0 , 0 , 0 , 0 , 0 , 0 , 0 ,255 },
  { 255,255,255,255,255,255,255,255,255,255,255, 0 ,255,255,255,255, 0 , 0 , 0 , 0 , 0 , 0 ,255 },
  { 255,255,255,255,255,255,255,255,255,255,255, 0 ,255,255,255,255, 1 , 0 , 0 , 4 , 4 , 4 ,255 },
  { 255,255,255,255,255,255,255,255,255,255,255, 0 ,255,255,255,255,255,255,255, 4 , 4 , 4 ,255 },
  { 255, 3 , 3 ,255,255,255,255,255,255,255,255, 0 ,255,255,255,255,255,255,255, 4 , 4 , 4 ,255 },
  { 255, 3 , 3 , 3 ,255,255,255,255,255,255,255, 0 ,255,255,255,255,255,255,255, 4 , 4 , 4 ,255 },
  { 255, 3 , 3 , 3 , 3 ,255,255,255,255,255,255, 0 ,255,255,255,255,255,255,255, 4 , 4 , 4 ,255 },
  { 255,255,255,255, 3 ,255,255,255,255,255,255, 0 ,255,255,255,255,255,255,255, 4 , 4 , 4 ,255 },
  { 255,255,255,255, 3 , 3 ,255,255,255,255,255, 0 ,255,255,255,255,255,255,255, 4 , 4 , 4 ,255 },
  { 255,255,255,255, 3 , 3 ,255,255,255,255,255, 0 ,255,255,255,255,255,255,255, 4 , 4 , 4 ,255 },
  { 255,255,255,255, 3 , 3 , 3 , 0 , 0 , 0 , 0 , 0 ,255,255,255,255,255,255,255,255,255,255,255 },
  { 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255 },
  { 255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255,255 }
  
};


int targetArrived = 0; // 1 = at target, 0 = not at target
int old_targetArrived = 0;  // 1 = at target, 0 = not at target 
int roomArrived = 0;  // 1 = in room, 0 = not in target room
int targetRoom = 1;  // 1 = dock, 2 = kitchen, 3 = family room, 4 = living room
int old_targetRoom = 1;
int currentRoom = 1;  //  0 = no room, 1 = dock, 2 = kitchen, 3 = family room, 4 = living room
int leds_cnt = 99;
int leds_state = 0;
int leds_on = 1;
int turn_angle = 0;
int turn_dir = 1;
int turning = 0;
int backing_up = 0;
int UserButtonPressed = 0;
int RobotIsOn = 0;
int headingDesired;  // temp heading variable used in rotate functions
int reading; // temp heading variable used in checkCompass
int sharpIRRight = 0;
int sharpIRLeft = 0;
int turnDirection = 0;
int absTurnDirection = 0;
int headingDiff = 0;
unsigned long sonarMinTemp;
unsigned long sonarMin;
unsigned long sonarReading1 = 0;
unsigned long sonarReading2 = 0;
unsigned long sonarReading3 = 0;

int speakCountJoke = 0;    // These are the counters that cycle through phrases for the WaveShield
int speakCountOffer = 0;
int speakCountObstacle = 0;
int speakCountComment = 0;
int speakCountWelcome = 0;
int speakCountDance = 0;
int speakCountJokeAnnounce = 0;
int speakCountDanceAnnounce = 0;

int roomLocation = 0;

int currentCylonMode = 1;
int dancing = 0;
char speakString[10];   // this is the string variable used to send the filename to the waveshield

void setup() {
  
  // Sonar setup
  pinMode(InitSonarRead, OUTPUT); // set digital pin to an input for Sonar
  digitalWrite(InitSonarRead, LOW);
  delay(500);
  digitalWrite(InitSonarRead, HIGH);
  delayMicroseconds(25);
  digitalWrite(InitSonarRead, LOW);
  //pinMode(InitSonarRead, INPUT); // set digital pin to an input for Sonar
  delay(500);
  
  // Set I/O pins
  
  pinMode(MoveSwitch, INPUT);  // set digital pin to an input for the Move Switch
  digitalWrite(MoveSwitch, HIGH);  // pull input pin to +5V
  pinMode(ModeSwitch, INPUT);  // set digital pin to an input for the Quiet Switch
  digitalWrite(ModeSwitch, HIGH);  // pull input pin to +5V
  pinMode(CylonReset, OUTPUT);  // set digital pin to an input for the Cylon Reset line
  digitalWrite(CylonReset, HIGH);  // pull output pin to +5V
  pinMode(PushButtonJoke, INPUT);  // set digital pin to an input for the Move Switch
  digitalWrite(PushButtonJoke, HIGH);  // pull input pin to +5V
  pinMode(PushButtonDance, INPUT);  // set digital pin to an input for the Quiet Switch
  digitalWrite(PushButtonDance, HIGH);  // pull input pin to +5V


  pinMode(RotarySwitch1, INPUT);  // set digital pin to an input for the rotary switch
  digitalWrite(RotarySwitch1, HIGH);  // pull input pin to +5V
  pinMode(RotarySwitch2, INPUT);  // set digital pin to an input for the rotary switch
  digitalWrite(RotarySwitch2, HIGH);  // pull input pin to +5V
  pinMode(RotarySwitch3, INPUT);  // set digital pin to an input for the rotary switch
  digitalWrite(RotarySwitch3, HIGH);  // pull input pin to +5V
  pinMode(RotarySwitch4, INPUT);  // set digital pin to an input for the rotary switch
  digitalWrite(RotarySwitch4, HIGH);  // pull input pin to +5V

  pinMode(UserButton, INPUT);  // set digital pin to an input for the user button
  digitalWrite(UserButton, HIGH);  // pull input pin to +5V
  pinMode(RobotPwrToggle, OUTPUT); // set digital pin to an output to turn on Create
  pinMode(RobotPowerSense, INPUT); // set digital pin to an input to sense Create power
    
  Serial.begin(57600);
  Serial1.begin(57600);	// opens serial port to Create, sets data rate to 57600
  Timer3.initialize(1000000);         // initialize timer3, and set a 1 second period
  Timer3.attachInterrupt(callback);  // attaches callback() as a timer overflow interrupt
  //MsTimer2::set(1000, delayCountFunction);   // attaches delayCountFunction() to an interrupt timer at 1000 ms (1 sec)
  //MsTimer2::start();    // start MsTimer2
  
  
  randomSeed(analogRead(0)); // make the random function more random

  powerOnRobot();
  delay(250);
  Serial1.write(CmdStart);
 
  defineSongs();
  Serial1.write(CmdControl);
  Serial1.write(CmdFull);

  // Stop just as a precaution
  drive(0, RadStraight);

  // Play the reset song and wait while it plays
  delay(250);
  Serial1.write(CmdPlay);
  Serial1.write(RESET_SONG);
  updateSensors();
  delay(750);
  
  // Cylon
  // Check how arduino was last reset
  if(bit_is_set(MCUSR, PORF))
    currentCylonMode = 1; 
  else if(bit_is_set(MCUSR, EXTRF))
    currentCylonMode = EEPROM.read(0);
 
  if(currentCylonMode > 11)
    currentCylonMode = 1; 
  
  cylonModeChange(1);  // default cylon mode
  
  //Compass
  Wire.begin();                // join i2c bus (address optional for master) 
  //Wire.beginTransmission(compassAddress);  // transmit to device               
  //Wire.send('O');          // command sensor to reset  
  //Wire.endTransmission();  // stop transmitting 


  //LCD
  lcd.begin(16,2);
  lcd.clear();
  lcd.print("Hello,my name is");
  lcd.setCursor(0, 1);
  lcd.print("   Belvedere");
  
  // Bowtie Servo
  servoBowTie.attach(6);
  servoBowTie.write(1500);
  delay(500);
  servoBowTie.detach();
  
  //WaveShield
  //  if (!card.init(true)) { //play with 4 MHz spi if 8MHz isn't working for you
  if (!card.init()) {         //play with 8 MHz spi (default faster!)  
    putstring_nl("Card init. failed!");  // Something went wrong, lets print out why
    sdErrorCheck();
    while(1);                            // then 'halt' - do nothing!
  }
  
  // enable optimize read - some cards may timeout. Disable if you're having problems
  card.partialBlockRead(true);
 
// Now we will look for a FAT partition!
  uint8_t part;
  for (part = 0; part < 5; part++) {     // we have up to 5 slots to look in
    if (vol.init(card, part)) 
      break;                             // we found one, lets bail
  }
  if (part == 5) {                       // if we ended up not finding one  :(
    putstring_nl("No valid FAT partition!");
    sdErrorCheck();      // Something went wrong, lets print out why
    while(1);                            // then 'halt' - do nothing!
  }
  
  // Lets tell the user about what we found
  putstring("Using partition ");
  Serial.print(part, DEC);
  putstring(", type is FAT");
  Serial.println(vol.fatType(),DEC);     // FAT16 or FAT32?
  
  // Try to open the root directory
  if (!root.openRoot(vol)) {
    putstring_nl("Can't open root dir!"); // Something went wrong,
    while(1);                             // then 'halt' - do nothing!
  }
  
  // Whew! We got past the tough parts.
  putstring_nl("Ready!");                         // then 'halt' - do nothing!
  
}


void loop() 
{
  // These are the counters that cycle through phrases for the WaveShield
  speakCountJoke = random(0, JokeEnd - JokeStart);    
  speakCountOffer = random(0, OfferEnd - OfferStart);
  speakCountObstacle = random(0, ObstacleEnd - ObstacleStart);
  speakCountComment = random(0, CommentEnd - CommentStart);
  speakCountWelcome = random(0, WelcomeEnd - WelcomeStart);
  speakCountDance = random(0, DanceEnd - DanceStart);
  speakCountJokeAnnounce = random(0, JokeAnnounceEnd - JokeAnnounceStart);
  speakCountDanceAnnounce = random(0, DanceAnnounceEnd - DanceAnnounceStart);
  
  updateSensors();


  
  if(!(digitalRead(UserButton)))
  {
    // Play start song and wait
    Serial1.write(CmdPlay);
    Serial1.write(START_SONG);
    delay(2913);
    updateSensors();
    //checkCompass();
    
    if((!(digitalRead(MoveSwitch))) && (digitalRead(ModeSwitch)))
      adjustHeading();
    
    // Drive around until a button or unsafe condition is detected
    while(!(!(digitalRead(UserButton)))
      && (!sensors[SenCliffL])
      && (!sensors[SenCliffFL])
      && (!sensors[SenCliffFR])
      && (!sensors[SenCliffR])
      && (!sensors[SenChAvailable])
      )
    {
      
      if(digitalRead(ModeSwitch))  // if mode switch is set to navigate
      {
       
        
        old_targetArrived = targetArrived;
        
        checkLocation();
        
        if((old_targetArrived != targetArrived) && (targetArrived))
          if(!(digitalRead(MoveSwitch)))
            adjustHeading();
        
        if((delayOfferCount > DelayOffer) && (targetArrived))
          {
            delayOfferCount = 0;
            if(!(wave.isplaying))
            {  
              spinBowTie();
              speak(Offer);
            }  
          }
        
        if((delayCommentCount > DelayComment) && (targetArrived))
          {
            delayCommentCount = 0;
            if(!(wave.isplaying))
              {  
              spinBowTie();
              speak(Comment);
            }  
          }
        
  
        if((old_targetArrived != targetArrived) || !(targetArrived))
          timeStationary = 0;
        
  
        if(timeStationary > TimeStationary)
          {
          roomLocation++;
          if(roomLocation > 1)
            roomLocation = 0;
          timeStationary = 0;
          }
          
        if(old_targetRoom != targetRoom)
        {
          drive(0,RadStraight);
          delay(1000);
          checkLocation();
        } 
  
  
        if(!(targetArrived))
          {
            //lcd.clear();
            driveToTarget();
          }
        else
          {
            drive(0,RadStraight);
            cylonModeChange(CylonStandard);
            //lcd.clear();
            //lcd.print("AT TARGET!  ");
            //lcd.print(roomLocation);
          }
      
      }
          
      else  // if mode switch is set to roam
      {
      
      
      if(delayOfferCount > DelayOffer)
          {
            delayOfferCount = 0;
            if(!(wave.isplaying))
            {  
              spinBowTie();
              speak(Offer);
            }  
          }
        
      if(delayCommentCount > DelayComment)
          {
            delayCommentCount = 0;
            if(!(wave.isplaying))
            {  
              spinBowTie();
              speak(Comment);
            }  
          }
        
        if(timeStationary > TimeStationary)
          {
          while(wave.isplaying)
          {}
          rotateCW((random(40,180)),200);
          moveStraightRoam(1000,RoamVelocity);
          timeStationary = 0;
          }
      
        
      }
      
        
      if(!(digitalRead(PushButtonJoke)))
        {
          spinBowTie();
          speak(JokeAnnounce);
          while(wave.isplaying)
          {}
          delay(1000);  
          speak(Joke);
          while(wave.isplaying)
          {}  
          delay(500);
          spinBowTie();
        }
        
        if(!(digitalRead(PushButtonDance)))
          {
            spinBowTie();
            dance();
          }
      
      delay(20);
      updateSensors();

    }

    // Stop driving
    drive(0, RadStraight);

    // Play end song and wait
    updateSensors();
    delay(500);
    Serial1.write(CmdPlay);
    Serial1.write(END_SONG);
    updateSensors();
    delay(2438);

  }
}


// MsTimerThree interrupt is called once per second
void callback()
{
  countFlag1++;
  
  //delayJokeCount++;    
  delayOfferCount++;
  delayCommentCount++;
  delayObstacleCount++;
  //delayDanceCount++;
  
  timeStationary++;
  
}


void updateSensors(void)
{

  while((millis() - updateDelayCheck) < 20)
  {}
  
  Serial1.write(CmdSensors);
  Serial1.write(6);
  sensors_index = 0;
  sensors_flag = 1;

  while(sensors_flag)
  {
    if(Serial1.available() > 0)
    {   
      sensors[sensors_index] = Serial1.read();
      sensors_index = sensors_index + 1;
      if(sensors_index >= Sen6Size)
        sensors_flag = 0;
    }
  }

  // Update running totals of distance and angle
  distance += (int)((sensors[SenDist1] << 8) | sensors[SenDist0]);
  angle += (int)((sensors[SenAng1] << 8) | sensors[SenAng0]);
  
  updateDelayCheck = millis();

}


void powerOnRobot(void)
{
  // If Create's power is off, turn it on
  //RobotIsOn = digitalRead(RobotPowerSense);

  if(!(digitalRead(RobotPowerSense)))
  {
    while(!RobotIsOn)
    {
      digitalWrite(RobotPwrToggle, LOW);
      delay(500);  // Delay in this state
      digitalWrite(RobotPwrToggle, HIGH);  // Low to high transition to toggle power
      delay(100);  // Delay in this state
      digitalWrite(RobotPwrToggle, LOW);
      RobotIsOn = digitalRead(RobotPowerSense);
    }
    delay(3500);  // Delay for startup
  }
}



// Send Create drive commands in terms of velocity and radius
void drive(int16_t velocity, int16_t radius)
{
  Serial1.write(CmdDrive);
  Serial1.write((uint8_t)((velocity >> 8) & 0x00FF));
  Serial1.write((uint8_t)(velocity & 0x00FF));
  Serial1.write((uint8_t)((radius >> 8) & 0x00FF));
  Serial1.write((uint8_t)(radius & 0x00FF));
}


void driveDirect(int16_t rightVelocity, int16_t leftVelocity)

{
  Serial1.write(CmdDriveWheels);
  Serial1.write((uint8_t)((rightVelocity >> 8) & 0x00FF));
  Serial1.write((uint8_t)(rightVelocity & 0x00FF));
  Serial1.write((uint8_t)((leftVelocity >> 8) & 0x00FF));
  Serial1.write((uint8_t)(leftVelocity & 0x00FF));
}

// Define songs to be played later
void defineSongs(void)
{
  // Reset song
  Serial1.write(CmdSong);
  Serial1.write(RESET_SONG);
  Serial1.write(4);
  Serial1.write(60);
  Serial1.write(6);
  Serial1.write(72);
  Serial1.write(6);
  Serial1.write(84);
  Serial1.write(6);
  Serial1.write(96);
  Serial1.write(6);

  // Start song
  Serial1.write(CmdSong);
  Serial1.write(START_SONG);
  Serial1.write(6);
  Serial1.write(69);
  Serial1.write(18);
  Serial1.write(72);
  Serial1.write(12);
  Serial1.write(74);
  Serial1.write(12);
  Serial1.write(72);
  Serial1.write(12);
  Serial1.write(69);
  Serial1.write(12);
  Serial1.write(77);
  Serial1.write(24);

  // Bump song
  Serial1.write(CmdSong);
  Serial1.write(BUMP_SONG);
  Serial1.write(2);
  Serial1.write(74);
  Serial1.write(12);
  Serial1.write(59);
  Serial1.write(24);

  // End song
  Serial1.write(CmdSong);
  Serial1.write(END_SONG);
  Serial1.write(5);
  Serial1.write(77);
  Serial1.write(18);
  Serial1.write(74);
  Serial1.write(12);
  Serial1.write(72);
  Serial1.write(12);
  Serial1.write(69);
  Serial1.write(12);
  Serial1.write(65);
  Serial1.write(24);
}

// Switch the baud rate on both Create

void baud(uint8_t baud_code)
{
  Serial1.write(CmdBaud);
  Serial1.write(baud_code);

  delay(200);
}



void checkCompass(void)
{
  // step 1: instruct sensor to read echoes 
  Wire.beginTransmission(compassAddress);  // transmit to device
  // the address specified in the datasheet is 66 (0x42) 
  // but i2c adressing uses the high 7 bits so it's 33 
  Wire.send('A');          // command sensor to measure angle  
  Wire.endTransmission();  // stop transmitting 

  // step 2: wait for readings to happen 
  delay(7);               // datasheet suggests at least 6000 microseconds 

  // step 3: request reading from sensor 
  Wire.requestFrom(compassAddress, 2);  // request 2 bytes from slave device #33 

  // step 4: receive reading from sensor 
  if(2 <= Wire.available())     // if two bytes were received 
  { 
    reading = Wire.receive();   // receive high byte (overwrites previous reading) 
    reading = reading << 8;     // shift high byte to be high 8 bits 
    reading += Wire.receive();  // receive low byte as lower 8 bits 
    reading /= 10;
    heading = reading;
  }
}

void updateDistanceSensors(void)
{
  int i = 0;
  unsigned long sonarSum1 = 0;
  unsigned long sonarSum2 = 0;
  unsigned long sonarSum3 = 0;
  
  sharpIRRight = analogRead(0);
  sharpIRLeft = analogRead(1);
  
  digitalWrite(InitSonarRead, HIGH);
  delayMicroseconds(25);
  digitalWrite(InitSonarRead, LOW);
  
  for(i = 0; i < 10; i++)
  {
    sonarReading1 = analogRead(2);
    sonarReading2 = analogRead(3);
    sonarReading3 = analogRead(4);
    
    sonarSum1 = sonarSum1 + sonarReading1;
    sonarSum2 = sonarSum2 + sonarReading2;
    sonarSum3 = sonarSum3 + sonarReading3;
    
    delay(10);
  }
  
  sonarMinTemp = min(sonarSum1,sonarSum2);
  sonarMin = (min(sonarSum3, sonarMinTemp)) / 10;
}


void moveStraight(int distTemp, int velocity)
{
  
  distance = 0;
  int velocityTemp = velocity;
  
  cylonModeChange(CylonRandomMulti);
  
  while(distance < distTemp)
  {
    if(countFlag1 > 0)
      {
        //lcd.clear();
        //lcd.print(millis() - loopDelayCheck);
        countFlag1 = 0;
      }
    loopDelayCheck = millis();
    
    updateSensors();
    
    updateDistanceSensors();
    
    //lcd.clear();
    //lcd.print(sonarMin);
    
//    if(sensors[SenIRChar] == 10)
//      {
//        updatePositionFlag = 1;
//        robot_x = 10;
//        robot_y = 11;
//        break;
//      }
//      
//    if(sensors[SenIRChar] == 11)
//      {
//        updatePositionFlag = 1;
//        robot_x = 14;
//        robot_y = 8;
//        break;
//      }
    
    if((sonarMin < SonarThreshold1) || (sensors[SenBumpDrop] & BumpEither))
      {
        drive(0, RadStraight);
        velocityTemp = 0;
      }
    else if(sonarMin < SonarThreshold3)
      velocityTemp = (sonarMin * 5) - 25 ;
    else
      velocityTemp = velocity;  
        
    if((sharpIRLeft > SharpIRThresholdLeft) && (sharpIRRight > SharpIRThresholdRight))
      drive(0, RadStraight);
    else if(sharpIRRight > SharpIRThresholdRight)
      drive(velocityTemp, WallAvoidRadius);
    else if(sharpIRLeft > SharpIRThresholdLeft)
      drive(velocityTemp, -WallAvoidRadius);
    else
      drive(velocityTemp, RadStraight);
         
    if((velocityTemp == 0) && !(wave.isplaying))
    {
      speak(Obstacle);
      // Play the bump song
      Serial1.write(CmdPlay);
      Serial1.write(BUMP_SONG);    
    }
    
    //delay(100);
  }

}


void moveStraightRoam(int distTemp, int velocity)
{
  cylonModeChange(CylonRandomMulti);
  distance = 0;
  int velocityTemp = velocity;
  
  while(distance < distTemp)
  {
    if(countFlag1 > 0)
      {
        //lcd.clear();
        //lcd.print(millis() - loopDelayCheck);
        countFlag1 = 0;
      }
    loopDelayCheck = millis();
    
    updateSensors();
    
    updateDistanceSensors();
    
    
    if((sonarMin < SonarThreshold1) || (sensors[SenBumpDrop] & BumpEither) || (sensors[SenBumpDrop] & BumpBoth))
      {
        if(sensors[SenBumpDrop] & BumpLeft)
          turnDirection = 1;
        else
          turnDirection = 0;
        
        drive(0, RadStraight);
        delay(500);
        if(!(wave.isplaying))
          {
          speak(Obstacle);
          // Play the bump song
          Serial1.write(CmdPlay);
          Serial1.write(BUMP_SONG);    
          }
        drive(-50,RadStraight);
        delay(200);
        if(turnDirection)
          rotateCW((random(40,180)),200);
        else
          rotateCCW((random(40,180)),200);
      }
    
    else if(sonarMin < SonarThreshold3)
      velocityTemp = (sonarMin * 5) - 25 ;
    else
      velocityTemp = velocity;  
        
    if((sharpIRLeft > SharpIRThresholdLeft) && (sharpIRRight > SharpIRThresholdRight))
      {
        drive(0, RadStraight);
        delay(1000);
        if(!(wave.isplaying))
          {
          speak(Obstacle);
          // Play the bump song
          Serial1.write(CmdPlay);
          Serial1.write(BUMP_SONG);    
          }
        drive(-50,RadStraight);
        delay(500);
        rotateCW((random(40,180)),200);
      }
    else if(sharpIRRight > SharpIRThresholdRight)
      drive(velocityTemp, WallAvoidRadius);
    else if(sharpIRLeft > SharpIRThresholdLeft)
      drive(velocityTemp, -WallAvoidRadius);
    else
      drive(velocityTemp, RadStraight);
         
    
    //delay(50);
  }

  drive(0, RadStraight);

}

void moveStraightSimple(int distTemp, int velocity)
{
  
  distance = 0;

  while(abs(distance) < abs(distTemp))
  {
  updateSensors();
  
  if(sensors[SenBumpDrop] & BumpEither)
    {
      drive(0, RadStraight);
      // Play the bump song
      Serial1.write(CmdPlay);
      Serial1.write(BUMP_SONG);
    }
  else
    drive(velocity, RadStraight);  
  
  delay(100);
  }
 
  drive(0,RadStraight);
 
}


void rotateCW(int turnAngle, int velocity)
{
  
  if(digitalRead(MoveSwitch))
  {
    //lcd.clear();
    //lcd.print("Using Encoders");
    angle = 0;
    drive(0,RadStraight);
    if(dancing == 0)
      delay(500);
    
    drive(velocity, RadCW);  
  
    while((-angle) < (turnAngle - TurnOffset))
    {
      updateSensors();
      delay(100);
    }
    drive(0, RadStraight);
    if(dancing == 0)
      delay(250);
  }
  else
  {
    drive(0,RadStraight);
    if(dancing == 0)
      delay(500);
    checkCompass();
    //lcd.clear();
    //lcd.print("Using Compass");
  
    headingDesired = heading + turnAngle;
    if(headingDesired > 359)
      headingDesired = headingDesired - 360;
  
    rotateAbsolute(headingDesired, velocity);

    drive(0, RadStraight);
    if(dancing == 0)
      delay(250); 
  }

}


void rotateCCW(int turnAngle, int velocity)
{
  if(digitalRead(MoveSwitch))
  {
    //lcd.clear();
    //lcd.print("Using Encoders");
    angle = 0;
    drive(0,RadStraight);
    if(dancing == 0)
      delay(500);
    drive(velocity, RadCCW);  
  
    while(angle < (turnAngle - TurnOffset))
    {
      updateSensors();
      delay(100);
    }
    drive(0, RadStraight);
    if(dancing == 0)
      delay(250);
  }
  else
  {
    drive(0,RadStraight);
    if(dancing == 0)
      delay(500);
    checkCompass();
    //lcd.clear();
    //lcd.print("Using Compass");
  
    headingDesired = heading - turnAngle;
    if(headingDesired < 0)
      headingDesired = headingDesired + 360;
  
    rotateAbsolute(headingDesired, velocity);

    drive(0, RadStraight);
    if(dancing == 0)
      delay(250); 
  }
  
}

void rotateAbsolute(int goal, int velocity)
{
  int d;
  int e;
  
  if(digitalRead(MoveSwitch))
  {
    //lcd.clear();
    //lcd.print("Encoders --- Can't do Absolute Move");
    drive(0,RadStraight);
    while(1)
    {}
  }
  else
  {
    
    drive(0,RadStraight);
    //if(dancing == 0)
    //  delay(500);
    
    if(checkTurnDirection(goal))    // returns 0 for CW, 1 for CCW  
      drive(velocity, RadCCW);
    else
      drive(velocity, RadCW);
    
    while(1)    
    {
      e = getHeadingError(goal);
      if (abs(e) < HeadingThreshold2) break;  
    }
    
    drive(0, RadStraight);
    checkCompass();
    //if(dancing == 0)
    //  delay(250);
    //lcd.clear();
    //lcd.print(heading); 
  }

}


int checkTurnDirection(int goal)   // Return 0 if turn direction should be CW, Return 1 if turn direction should be CCW
{
  checkCompass();
  int diff = goal - heading;
  
  if(((diff > 0) && (diff < 180)) || ((diff < 0) && (diff < -180)))
    return 0;   // CW
  else
    return 1;   // CCW
  
}


int getHeadingError(int d0) 
{ 
  int e; 
  checkCompass();
  e = heading - d0; 
  /* handle wrap-around */ 
  if (e < -180) 
    e = e + 360; 
  if (e > 180) 
    e = e - 360; 
  return e; 
} 

void adjustHeading(void)
{
    
  if((target_x == Dock_x) && (target_y == Dock_y))
  {
    if(newState == 1)
      rotateAbsolute(DockCompassHeading1,20);
    else if(newState == 2)
      rotateAbsolute(DockCompassHeading2,20);
    else if(newState == 3)
      rotateAbsolute(DockCompassHeading3,20);
    else if(newState == 4)
      rotateAbsolute(DockCompassHeading4,20);
  }
  else if((target_x == Kitchen1_x) && (target_y == Kitchen1_y))
  {
    if(newState == 1)
      rotateAbsolute(Kitchen1CompassHeading1,20);
    else if(newState == 2)
      rotateAbsolute(Kitchen1CompassHeading2,20);
    else if(newState == 3)
      rotateAbsolute(Kitchen1CompassHeading3,20);
    else if(newState == 4)
      rotateAbsolute(Kitchen1CompassHeading4,20);
  }  
  else if((target_x == Kitchen2_x) && (target_y == Kitchen2_y))
  {
    if(newState == 1)
      rotateAbsolute(Kitchen2CompassHeading1,20);
    else if(newState == 2)
      rotateAbsolute(Kitchen2CompassHeading2,20);
    else if(newState == 3)
      rotateAbsolute(Kitchen2CompassHeading3,20);
    else if(newState == 4)
      rotateAbsolute(Kitchen2CompassHeading4,20);
  }  
  else if((target_x == FamilyRoom1_x) && (target_y == FamilyRoom1_y))
  {
    if(newState == 1)
      rotateAbsolute(FamilyRoom1CompassHeading1,20);
    else if(newState == 2)
      rotateAbsolute(FamilyRoom1CompassHeading2,20);
    else if(newState == 3)
      rotateAbsolute(FamilyRoom1CompassHeading3,20);
    else if(newState == 4)
      rotateAbsolute(FamilyRoom1CompassHeading4,20);
  }  
  else if((target_x == FamilyRoom2_x) && (target_y == FamilyRoom2_y))
  {
    if(newState == 1)
      rotateAbsolute(FamilyRoom2CompassHeading1,20);
    else if(newState == 2)
      rotateAbsolute(FamilyRoom2CompassHeading2,20);
    else if(newState == 3)
      rotateAbsolute(FamilyRoom2CompassHeading3,20);
    else if(newState == 4)
      rotateAbsolute(FamilyRoom2CompassHeading4,20);
  }  
  else if((target_x == LivingRoom1_x) && (target_y == LivingRoom1_y))
  {
    if(newState == 1)
      rotateAbsolute(LivingRoom1CompassHeading1,20);
    else if(newState == 2)
      rotateAbsolute(LivingRoom1CompassHeading2,20);
    else if(newState == 3)
      rotateAbsolute(LivingRoom1CompassHeading3,20);
    else if(newState == 4)
      rotateAbsolute(LivingRoom1CompassHeading4,20);
  }  
  else if((target_x == LivingRoom2_x) && (target_y == LivingRoom2_y))
  {
    if(newState == 1)
      rotateAbsolute(LivingRoom2CompassHeading1,20);
    else if(newState == 2)
      rotateAbsolute(LivingRoom2CompassHeading2,20);
    else if(newState == 3)
      rotateAbsolute(LivingRoom2CompassHeading3,20);
    else if(newState == 4)
      rotateAbsolute(LivingRoom2CompassHeading4,20);
  }
  else
  {
    if(newState == 1)
      rotateAbsolute(CompassHeading1,20);
    else if(newState == 2)
      rotateAbsolute(CompassHeading2,20);
    else if(newState == 3)
      rotateAbsolute(CompassHeading3,20);
    else if(newState == 4)
      rotateAbsolute(CompassHeading4,20);  
  }  
}



void checkLocation(void)
{
  if(!(digitalRead(RotarySwitch1)))
  {
    old_targetRoom = targetRoom;    
    targetRoom = Dock;
    old_target_x = target_x;
    old_target_y = target_y;
    target_x = Dock_x;
    target_y = Dock_y;
    waveMap[old_target_x][old_target_y] = Nothing;
    waveMap[target_x][target_y] = Target;
  }


  if(!(digitalRead(RotarySwitch2)))
  {
    old_targetRoom = targetRoom;
    targetRoom = Kitchen;
    old_target_x = target_x;
    old_target_y = target_y;
    if(roomLocation == 0)
      {
      target_x = Kitchen1_x;
      target_y = Kitchen1_y;
      }
    else
      {
      target_x = Kitchen2_x;
      target_y = Kitchen2_y;
      }
    
    waveMap[old_target_x][old_target_y] = Nothing;
    waveMap[target_x][target_y] = Target;

  } 


  if(!(digitalRead(RotarySwitch3)))
  {
    old_targetRoom = targetRoom;
    targetRoom = FamilyRoom;
    old_target_x = target_x;
    old_target_y = target_y;
    if(roomLocation == 0)
      {
      target_x = FamilyRoom1_x;
      target_y = FamilyRoom1_y;
      }
    else
      {
      target_x = FamilyRoom2_x;
      target_y = FamilyRoom2_y;
      }
    
    waveMap[old_target_x][old_target_y] = Nothing;
    waveMap[target_x][target_y] = Target;
  } 


  if(!(digitalRead(RotarySwitch4)))
  {
    old_targetRoom = targetRoom;    
    targetRoom = LivingRoom;
    old_target_x = target_x;
    old_target_y = target_y;
    if(roomLocation == 0)
      {
      target_x = LivingRoom1_x;
      target_y = LivingRoom1_y;
      }
    else
      {
      target_x = LivingRoom2_x;
      target_y = LivingRoom2_y;
      }
    waveMap[old_target_x][old_target_y] = Nothing;
    waveMap[target_x][target_y] = Target;
  }

  currentRoom = roomMap[robot_x][robot_y];

  if(waveMap[robot_x][robot_y] == Target)
    targetArrived = 1;
  else
    targetArrived = 0;

  if(currentRoom == targetRoom)
    roomArrived = 1;
  else
    roomArrived = 0;
    
}

void driveToTarget(void)
{

  newState = propogateWavefront(robot_x, robot_y, target_x, target_y);

  while((newState != oldState) && (newState != 0)) 
  {
    //if not pointed in the right direction, rotate
    if(abs(oldState - newState) == 2)
      rotateCW(180,RotateVelocity);
    else if(((oldState - newState) == -1) || ((oldState - newState) == 3))
      rotateCW(90,RotateVelocity);
    else if(((oldState - newState) == 1) || ((oldState - newState) == -3))
      rotateCCW(90,RotateVelocity);
  
//    if(newState == 1)
//      rotateAbsolute(CompassHeading1,RotateVelocity);
//    else if(newState == 2)
//      rotateAbsolute(CompassHeading2,RotateVelocity);
//    else if(newState == 3)
//      rotateAbsolute(CompassHeading3,RotateVelocity);
//    else if(newState == 4)
//      rotateAbsolute(CompassHeading4,RotateVelocity);

    //make new state the old state
    oldState = newState;

    //find new direction to move
    newState = propogateWavefront(robot_x, robot_y, target_x, target_y);

  }
  
  
  
  if(newState == 0)
  {
    Serial1.write(CmdPlay);
    Serial1.write(BUMP_SONG);
    delay(2500);
  }
  else
  {

    moveStraight(CellSize,DriveVelocity);  // move straight(distance,velocity) 

    // update robot location
    
    if(updatePositionFlag == 1)
      updatePositionFlag = 0;  
    else if(newState == 1)
      robot_x--;
    else if(newState == 2)
      robot_y++;
    else if(newState == 3)
      robot_x++;
    else if(newState == 4)
      robot_y--;  
  }

  // make new state the old state
  if(newState != 0)
    oldState = newState;

}


int propogateWavefront(int robot_x, int robot_y, int target_x, int target_y)
{


  //clear old wavefront
  unpropogate(robot_x, robot_y);

  waveCounter=0;//reset the counter for each run!
  while(waveCounter<50)//allows for recycling until robot is found
  {
    x=0;
    y=0;
    while((x < Max_x) && (y < Max_y))//while the map hasnt been fully scanned
    {
      //if this location is a wall or the goal, just ignore it
      if (waveMap[x][y] != Wall && waveMap[x][y] != Target)
      {	
        //a full trail to the robot has been located, finished!
        if (min_surrounding_node_value(x, y) < reset_min && waveMap[x][y]==Robot)
          return min_node_location;  //finished!
        //record a value in to this node
        else if (minimum_node != reset_min)//if this isnt here, 'nothing' will go in the location
            waveMap[x][y]=minimum_node+1;

      }

      //go to next node and/or row
      y++;

      if ((y==Max_y) && (x!=Max_x))
      {
        x++;
        y=0;
      }
    }

    waveCounter++;
  }
  return 0;
}

void unpropogate(int robot_x, int robot_y)  // clears old path to determine new path
{
  //stay within boundary
  for(x=0; x < Max_x; x++)
    for(y=0; y< Max_y; y++)
      if (waveMap[x][y] != Wall && waveMap[x][y] != Target) //if this location is something, just ignore it
          waveMap[x][y] = Nothing;//clear that space
  //store robot location in map
  waveMap[robot_x][robot_y]=Robot;

  //store target location in map
  waveMap[target_x][target_y]=Target;
}


void clear_target(void)
{	

  for(x=0; x < Max_x; x++)
    for(y=0; y< Max_y; y++)
      if (waveMap[x][y] != Robot && waveMap[x][y] != Wall)
        waveMap[x][y]=Nothing;
} 


int min_surrounding_node_value(int x, int y)
{
  minimum_node=reset_min;//reset minimum
  //down
  if(x < (Max_x - 1))//not out of boundary
    if  (waveMap[x+1][y] < minimum_node && waveMap[x+1][y] != Nothing)//find the lowest number node, and exclude empty nodes (0's)
    {
      minimum_node = waveMap[x+1][y];
      min_node_location=3;
    }

  //up
  if(x > 0)
    if  (waveMap[x-1][y] < minimum_node && waveMap[x-1][y] != Nothing)
    {
      minimum_node = waveMap[x-1][y];
      min_node_location=1;
    }

  //right
  if(y < (Max_y - 1))
    if  (waveMap[x][y+1] < minimum_node && waveMap[x][y+1] != Nothing)
    {
      minimum_node = waveMap[x][y+1];
      min_node_location=2;
    }

  //left
  if(y > 0)
    if(waveMap[x][y-1] < minimum_node && waveMap[x][y-1] != Nothing)
    {
      minimum_node = waveMap[x][y-1];
      min_node_location=4;
    }
  return minimum_node;

}

void printMap(void)
{
  for(x=0; x < Max_x; x++)
    for(y=0; y< Max_y; y++)
    {
      Serial.print(waveMap[x][y]);
      Serial.print("  ");
      if(y == (Max_y -1))
        Serial.println(" ");
    }

  Serial.println("  ");
  Serial.println("  ");
}


int freeRam(void)
{
  extern int  __bss_end; 
  extern int  *__brkval; 
  int free_memory; 
  if((int)__brkval == 0) {
    free_memory = ((int)&free_memory) - ((int)&__bss_end); 
  }
  else {
    free_memory = ((int)&free_memory) - ((int)__brkval); 
  }
  return free_memory; 
} 

void sdErrorCheck(void)
{
  if (!card.errorCode()) return;
  putstring("\n\rSD I/O error: ");
  Serial.print(card.errorCode(), HEX);
  putstring(", ");
  Serial.println(card.errorData(), HEX);
  while(1);
}

// Plays a full file from beginning to end with no pause.
void playcomplete(char *name) {
  // call our helper to find and play this name
  playfile(name);
  while (wave.isplaying) {
  // do nothing while its playing
  }
  // now its done playing
}

void playfile(char *name) {
  // see if the wave object is currently doing something
  if (wave.isplaying) {// already playing something, so stop it!
    wave.stop(); // stop it
  }
  // look in the root directory and open the file
  if (!f.open(root, name)) {
    putstring("Couldn't open file "); Serial.print(name); return;
  }
  // OK read the file and turn it into a wave object
  if (!wave.create(f)) {
    putstring_nl("Not a valid WAV"); return;
  }
  
  // ok time to play! start playback
  wave.play();
}

void speak(int phraseType)
{
 
  
  switch (phraseType) {
    case 1:  // speak Joke
      
      sprintf(speakString, "%d.WAV", JokeStart + speakCountJoke);
      playfile(speakString);
      if((JokeStart + speakCountJoke) == JokeEnd)
        speakCountJoke = 0;
      else
        speakCountJoke++;
      
      break;
    case 2:  // speak Offer
      
      sprintf(speakString, "%d.WAV", OfferStart + speakCountOffer);
      playfile(speakString);
      if((OfferStart + speakCountOffer) == OfferEnd)
        speakCountOffer = 0;
      else
        speakCountOffer++;
        
      break;
    case 3: // Speak Obstacle

      sprintf(speakString, "%d.WAV", ObstacleStart + speakCountObstacle);
      playfile(speakString);
      if((ObstacleStart + speakCountObstacle) == ObstacleEnd)
        speakCountObstacle = 0;
      else
        speakCountObstacle++;
        
      break;
    case 4:  // Speak Comment
      
      sprintf(speakString, "%d.WAV", CommentStart + speakCountComment);
      playfile(speakString);
      if((CommentStart + speakCountComment) == CommentEnd)
        speakCountComment = 0;
      else
        speakCountComment++;
        
      break; 
    case 5:  // Speak Welcome
      
      sprintf(speakString, "%d.WAV", WelcomeStart + speakCountWelcome);
      playfile(speakString);
      if((WelcomeStart + speakCountWelcome) == WelcomeEnd)
        speakCountWelcome = 0;
      else
        speakCountWelcome++;
        
      break;
   case 6:  // Dance music
      
      sprintf(speakString, "%d.WAV", DanceStart + speakCountDance);
      playfile(speakString);
      if((DanceStart + speakCountDance) == DanceEnd)
        speakCountDance = 0;
      else
        speakCountDance++;
        
      break;
  case 7:  // Joke Announce
      
      sprintf(speakString, "%d.WAV", JokeAnnounceStart + speakCountJokeAnnounce);
      playfile(speakString);
      if((JokeAnnounceStart + speakCountJokeAnnounce) == JokeAnnounceEnd)
        speakCountJokeAnnounce = 0;
      else
        speakCountJokeAnnounce++;
        
      break;
   case 8:  // Dance Announce
      
      sprintf(speakString, "%d.WAV", DanceAnnounceStart + speakCountDanceAnnounce);
      playfile(speakString);
      if((DanceAnnounceStart + speakCountDanceAnnounce) == DanceAnnounceEnd)
        speakCountDanceAnnounce = 0;
      else
        speakCountDanceAnnounce++;
        
      break;          
  }
  
}


// Change to one of 11 modes in the cylon microcontroller
void cylonModeChange(int mode) 
{
  if(mode == currentCylonMode)
    return;
  
  if(mode < currentCylonMode)
    mode = mode + 11;
  
  int i = 0;
  
  for(i = 0; i < (mode - currentCylonMode); i++)
  {
    digitalWrite(CylonReset, LOW);
    delay(10);
    digitalWrite(CylonReset, HIGH);
    delay(50);   
  }
  
  currentCylonMode = mode;
  EEPROM.write(0,currentCylonMode);
  delay(4);
}


void dance(void)
{
  dancing = 1;
  delay(500);
  speak(DanceAnnounce);
  while(wave.isplaying)
  {}
  delay(1000);
  long danceTime = millis();
  speak(Dance);
  delay(2000);
  
  while((millis() - danceTime) < 20000)
  {
    rotateCW(10,200);
    rotateCCW(20,200);
    rotateCW(20,200);
    rotateCCW(10,200);
    spinBowTie();
    moveStraightSimple(100,200);
    delay(100);
    moveStraightSimple(100,-200);
    delay(100);
    moveStraightSimple(100,200);
    delay(100);
    moveStraightSimple(100,-200);
    spinBowTie();
    
  }
  
  if (wave.isplaying)// already playing something, so stop it!
    wave.stop(); // stop it
  
  delay(2000);
  adjustHeading();
  dancing = 0;

}

void spinBowTie(void)
{
  int pos = 1500;
  servoBowTie.attach(6);
  
  for(pos = 1500; pos < 2000; pos += 40)  // goes from 70 degrees to 110 degrees 
    {                               // in steps of 1 degree 
      servoBowTie.write(pos);              // tell servo to go to position in variable 'pos' 
      delay(10);                       // waits 15ms for the servo to reach the position 
    } 
  for(pos = 2000; pos>=1000; pos-=40)     // goes from 110 degrees to 70 degrees 
    {                                
      servoBowTie.write(pos);              // tell servo to go to position in variable 'pos' 
      delay(10);                       // waits 15ms for the servo to reach the position 
    }
  for(pos = 1000; pos < 1500; pos += 40)  // goes from 70 degrees to 90 degrees 
    {                               // in steps of 1 degree 
      servoBowTie.write(pos);              // tell servo to go to position in variable 'pos' 
      delay(10);                       // waits 15ms for the servo to reach the position 
    } 

  servoBowTie.write(1500);
  delay(100);
  servoBowTie.detach();
  delay(100);
}
